@@ -207,6 +207,43 @@ static PS_WORD         reorder_word PT((PS_WORD));
 static char    *skipped_line = "% ps_io_fgetchars: skipped line";
 static char    *empty_string = "";
 
+static Boolean scan_boundingbox(int *bb, const char *line)
+{
+    char fllx[21], flly[21], furx[21], fury[21];
+    
+    if (sscanf (line, "%d %d %d %d",
+		&bb[LLX], &bb[LLY], &bb[URX], &bb[URY]) == 4)
+       return True;
+    
+    if (sscanf (line, "%20s %20s %20s %20s",
+		fllx, flly, furx, fury) == 4) {
+       float ffllx, fflly, ffurx, ffury;
+
+       ffllx = _spectre_strtod (fllx, NULL);
+       fflly = _spectre_strtod (flly, NULL);
+       ffurx = _spectre_strtod (furx, NULL);
+       ffury = _spectre_strtod (fury, NULL);
+       
+       bb[LLX] = ffllx;
+       bb[LLY] = fflly;
+       bb[URX] = ffurx;
+       bb[URY] = ffury;
+       
+       if (bb[LLX] > ffllx)
+          bb[LLX]--;
+       if (bb[LLY] > fflly)
+          bb[LLY]--;
+       if (bb[URX] < ffurx)
+          bb[URX]++;
+       if (bb[URY] < ffury)
+          bb[URY]++;
+       
+       return True;
+    }
+    
+    return False;
+}
+
 /*--------------------------------------------------*/
 
 /*
@@ -425,9 +462,8 @@ psscan(const char *filename, int scanstyle)
     if ((iscomment(line,"%!PS") || iscomment(line, "\004%!PS"))) {
       INFMESSAGE(found "PS-Adobe-" comment)
 
-      doc = (struct document *) PS_malloc(sizeof(struct document));
+      doc = (struct document *) PS_calloc(1, sizeof(struct document));
       CHECK_MALLOCED(doc);
-      memset(doc, 0, sizeof(struct document));
 
       /* ignore possible leading ^D */
       if (*line == '\004') {
@@ -462,9 +498,8 @@ psscan(const char *filename, int scanstyle)
 	   In a way, this makes sense, a program PostScript does not need
 	   the !PS at the beginning.
 	*/
-	doc = (struct document *) PS_malloc(sizeof(struct document));
+	doc = (struct document *) PS_calloc(1, sizeof(struct document));
 	CHECK_MALLOCED(doc);
-	memset(doc, 0, sizeof(struct document));
 	doc->ref_count = 1;
 	doc->filename = _spectre_strdup (filename);
 	doc->default_page_orientation = NONE;
@@ -502,31 +537,9 @@ psscan(const char *filename, int scanstyle)
 	    if (strcmp(text, "(atend)") == 0) {
 		bb_set = ATEND;
 	    } else {
-		if (sscanf(line+length("%%BoundingBox:"), "%d %d %d %d",
-			   &(doc->boundingbox[LLX]),
-			   &(doc->boundingbox[LLY]),
-			   &(doc->boundingbox[URX]),
-			   &(doc->boundingbox[URY])) == 4)
+		if (scan_boundingbox(doc->boundingbox,
+				line + length("%%BoundingBox:")))
 		    bb_set = 1;
-		else {
-		    float fllx, flly, furx, fury;
-		    if (sscanf(line+length("%%BoundingBox:"), "%f %f %f %f",
-			       &fllx, &flly, &furx, &fury) == 4) {
-			bb_set = 1;
-			doc->boundingbox[LLX] = fllx;
-			doc->boundingbox[LLY] = flly;
-			doc->boundingbox[URX] = furx;
-			doc->boundingbox[URY] = fury;
-			if (fllx < doc->boundingbox[LLX])
-			    doc->boundingbox[LLX]--;
-			if (flly < doc->boundingbox[LLY])
-			    doc->boundingbox[LLY]--;
-			if (furx > doc->boundingbox[URX])
-			    doc->boundingbox[URX]++;
-			if (fury > doc->boundingbox[URY])
-			    doc->boundingbox[URY]++;
-		    }
-		}
 	    }
 	} else if (orientation_set == NONE &&
 		   iscomment(line+2, "Orientation:")) {
@@ -592,16 +605,16 @@ psscan(const char *filename, int scanstyle)
 	    }
 	} else if (doc->nummedia == NONE &&
 		   iscomment(line+2, "DocumentMedia:")) {
-	    float w, h;
+	    char w[21], h[21];
 	    doc->media = (Media)
 			 PS_malloc(sizeof (MediaStruct));
             CHECK_MALLOCED(doc->media);
 	    doc->media[0].name = ps_gettext(line+length("%%DocumentMedia:"),
 					    &next_char);
 	    if (doc->media[0].name != NULL) {
-		if (sscanf(next_char, "%f %f", &w, &h) == 2) {
-		    doc->media[0].width = w + 0.5;
-		    doc->media[0].height = h + 0.5;
+		if (sscanf(next_char, "%20s %20s", w, h) == 2) {
+		    doc->media[0].width = _spectre_strtod (w, NULL) + 0.5;
+		    doc->media[0].height = _spectre_strtod (h, NULL) + 0.5;
 		}
 		if (doc->media[0].width != 0 && doc->media[0].height != 0)
 		    doc->nummedia = 1;
@@ -620,9 +633,9 @@ psscan(const char *filename, int scanstyle)
 		doc->media[doc->nummedia].name = ps_gettext(line+length("%%+"),
 							    &next_char);
 		if (doc->media[doc->nummedia].name != NULL) {
-		    if (sscanf(next_char, "%f %f", &w, &h) == 2) {
-			doc->media[doc->nummedia].width = w + 0.5;
-			doc->media[doc->nummedia].height = h + 0.5;
+		    if (sscanf(next_char, "%20s %20s", w, h) == 2) {
+		        doc->media[doc->nummedia].width = _spectre_strtod (w, NULL) + 0.5;
+			doc->media[doc->nummedia].height = _spectre_strtod (h, NULL) + 0.5;
 		    }
 		    if (doc->media[doc->nummedia].width != 0 &&
 			doc->media[doc->nummedia].height != 0) doc->nummedia++;
@@ -806,31 +819,9 @@ psscan(const char *filename, int scanstyle)
 		PS_free(cp);
 	    } else if (page_bb_set == NONE &&
 		       iscomment(line+2, "PageBoundingBox:")) {
-		if (sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
-			   &(doc->default_page_boundingbox[LLX]),
-			   &(doc->default_page_boundingbox[LLY]),
-			   &(doc->default_page_boundingbox[URX]),
-			   &(doc->default_page_boundingbox[URY])) == 4)
+		if (scan_boundingbox(doc->default_page_boundingbox,
+			    line+length("%%PageBoundingBox:")))
 		    page_bb_set = 1;
-		else {
-		    float fllx, flly, furx, fury;
-		    if (sscanf(line+length("%%PageBoundingBox:"), "%f %f %f %f",
-			       &fllx, &flly, &furx, &fury) == 4) {
-			page_bb_set = 1;
-			doc->default_page_boundingbox[LLX] = fllx;
-			doc->default_page_boundingbox[LLY] = flly;
-			doc->default_page_boundingbox[URX] = furx;
-			doc->default_page_boundingbox[URY] = fury;
-			if (fllx < doc->default_page_boundingbox[LLX])
-			    doc->default_page_boundingbox[LLX]--;
-			if (flly < doc->default_page_boundingbox[LLY])
-			    doc->default_page_boundingbox[LLY]--;
-			if (furx > doc->default_page_boundingbox[URX])
-			    doc->default_page_boundingbox[URX]++;
-			if (fury > doc->default_page_boundingbox[URY])
-			    doc->default_page_boundingbox[URY]++;
-		    }
-		}
 	    }
 	}
 	section_len += line_len;
@@ -936,31 +927,9 @@ psscan(const char *filename, int scanstyle)
 		PS_free(cp);
 	    } else if (page_bb_set == NONE &&
 		       iscomment(line+2, "PageBoundingBox:")) {
-		if (sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
-			   &(doc->default_page_boundingbox[LLX]),
-			   &(doc->default_page_boundingbox[LLY]),
-			   &(doc->default_page_boundingbox[URX]),
-			   &(doc->default_page_boundingbox[URY])) == 4)
+		if (scan_boundingbox(doc->default_page_boundingbox,
+			    line+length("%%PageBoundingBox:")))
 		    page_bb_set = 1;
-		else {
-		    float fllx, flly, furx, fury;
-		    if (sscanf(line+length("%%PageBoundingBox:"), "%f %f %f %f",
-			       &fllx, &flly, &furx, &fury) == 4) {
-			page_bb_set = 1;
-			doc->default_page_boundingbox[LLX] = fllx;
-			doc->default_page_boundingbox[LLY] = flly;
-			doc->default_page_boundingbox[URX] = furx;
-			doc->default_page_boundingbox[URY] = fury;
-			if (fllx < doc->default_page_boundingbox[LLX])
-			    doc->default_page_boundingbox[LLX]--;
-			if (flly < doc->default_page_boundingbox[LLY])
-			    doc->default_page_boundingbox[LLY]--;
-			if (furx > doc->default_page_boundingbox[URX])
-			    doc->default_page_boundingbox[URX]++;
-			if (fury > doc->default_page_boundingbox[URY])
-			    doc->default_page_boundingbox[URY]++;
-		    }
-		}
 	    }
 	}
 	section_len += line_len;
@@ -1103,38 +1072,10 @@ continuepage:
 		if (strcmp(text, "(atend)") == 0 || strcmp(text, "atend") == 0) {
 		    page_bb_set = ATEND;
 		} else {
-		    if (sscanf(line+length("%%PageBoundingBox:"), "%d %d %d %d",
-			    &(doc->pages[doc->numpages].boundingbox[LLX]),
-			    &(doc->pages[doc->numpages].boundingbox[LLY]),
-			    &(doc->pages[doc->numpages].boundingbox[URX]),
-			    &(doc->pages[doc->numpages].boundingbox[URY])) == 4) 
-		      {
-			if (page_bb_set == NONE) page_bb_set = 1;
-		      }
-		    else {
-			float fllx, flly, furx, fury;
-			if (sscanf(line+length("%%PageBoundingBox:"),
-				   "%f %f %f %f",
-				   &fllx, &flly, &furx, &fury) == 4) {
-			    if (page_bb_set == NONE) page_bb_set = 1;
-			    doc->pages[doc->numpages].boundingbox[LLX] = fllx;
-			    doc->pages[doc->numpages].boundingbox[LLY] = flly;
-			    doc->pages[doc->numpages].boundingbox[URX] = furx;
-			    doc->pages[doc->numpages].boundingbox[URY] = fury;
-			    if (fllx <
-				    doc->pages[doc->numpages].boundingbox[LLX])
-				doc->pages[doc->numpages].boundingbox[LLX]--;
-			    if (flly <
-				    doc->pages[doc->numpages].boundingbox[LLY])
-				doc->pages[doc->numpages].boundingbox[LLY]--;
-			    if (furx >
-				    doc->pages[doc->numpages].boundingbox[URX])
-				doc->pages[doc->numpages].boundingbox[URX]++;
-			    if (fury >
-				    doc->pages[doc->numpages].boundingbox[URY])
-				doc->pages[doc->numpages].boundingbox[URY]++;
-			}
-		    }
+		    if (scan_boundingbox(doc->pages[doc->numpages].boundingbox,
+				line+length("%%PageBoundingBox:")))
+			if(page_bb_set == NONE)
+			    page_bb_set = 1;
 		}
 	    }
 	}
@@ -1191,28 +1132,7 @@ continuepage:
 	    doc->begintrailer = position;
 	    section_len = line_len;
 	} else if (bb_set == ATEND && iscomment(line+2, "BoundingBox:")) {
-	    if (sscanf(line+length("%%BoundingBox:"), "%d %d %d %d",
-		       &(doc->boundingbox[LLX]),
-		       &(doc->boundingbox[LLY]),
-		       &(doc->boundingbox[URX]),
-		       &(doc->boundingbox[URY])) != 4) {
-		float fllx, flly, furx, fury;
-		if (sscanf(line+length("%%BoundingBox:"), "%f %f %f %f",
-			   &fllx, &flly, &furx, &fury) == 4) {
-		    doc->boundingbox[LLX] = fllx;
-		    doc->boundingbox[LLY] = flly;
-		    doc->boundingbox[URX] = furx;
-		    doc->boundingbox[URY] = fury;
-		    if (fllx < doc->boundingbox[LLX])
-			doc->boundingbox[LLX]--;
-		    if (flly < doc->boundingbox[LLY])
-			doc->boundingbox[LLY]--;
-		    if (furx > doc->boundingbox[URX])
-			doc->boundingbox[URX]++;
-		    if (fury > doc->boundingbox[URY])
-			doc->boundingbox[URY]++;
-		}
-	    }
+	    scan_boundingbox(doc->boundingbox, line + length("%%BoundingBox:"));
 	} else if (orientation_set == ATEND &&
 		   iscomment(line+2, "Orientation:")) {
 	    sscanf(line+length("%%Orientation:"), "%256s", text);
@@ -2280,6 +2200,42 @@ ps_read_doseps(fd,doseps)
     return doseps->ps_begin + doseps->ps_length;
 }
 
+int
+psgetpagebbox (const struct document *doc, int page, int *urx, int *ury, int *llx, int *lly)
+{
+   int new_llx = 0;
+   int new_lly = 0;
+   int new_urx = 0;
+   int new_ury = 0;
+
+   if ((page >= 0) &&
+       (doc->pages) &&
+       (doc->pages[page].boundingbox[URX] >
+	doc->pages[page].boundingbox[LLX]) &&
+       (doc->pages[page].boundingbox[URY] >
+	doc->pages[page].boundingbox[LLY])) {
+      /* use page bbox */
+      new_llx = doc->pages[page].boundingbox[LLX];
+      new_lly = doc->pages[page].boundingbox[LLY];
+      new_urx = doc->pages[page].boundingbox[URX];
+      new_ury = doc->pages[page].boundingbox[URY];
+   } else if ((doc->boundingbox[URX] > doc->boundingbox[LLX]) &&
+	      (doc->boundingbox[URY] > doc->boundingbox[LLY])) {
+      /* use doc bbox */
+      new_llx = doc->boundingbox[LLX];
+      new_lly = doc->boundingbox[LLY];
+      new_urx = doc->boundingbox[URX];
+      new_ury = doc->boundingbox[URY];
+   }
+
+   *llx = new_llx;
+   *lly = new_lly;
+   *urx = new_urx;
+   *ury = new_ury;
+
+   return (new_llx != 0 && new_lly != 0 && new_urx != 0 && new_ury != 0);
+}
+
 /* From Evince */
 #define DEFAULT_PAGE_SIZE 1
 
@@ -2301,7 +2257,7 @@ psgetpagebox (const struct document *doc, int page, int *urx, int *ury, int *llx
 	  * or the page bbox (if specified)	
 	  * or the bounding box	
 	  */
-	      if ((page >= 0) && (doc->numpages > (unsigned int)page) &&
+         if ((page >= 0) && (doc->numpages > (unsigned int)page) &&
 	     (doc->pages) && (doc->pages[page].media)) {
 	    new_pagesize = doc->pages[page].media - doc->media;
 	 } else if (doc->default_page_media != NULL) {
@@ -2323,25 +2279,7 @@ psgetpagebox (const struct document *doc, int page, int *urx, int *ury, int *llx
 
    /* Compute bounding box */
    if (doc && (doc->epsf || new_pagesize == -1)) {    /* epsf or bbox */
-      if ((page >= 0) &&
-	  (doc->pages) &&
-	  (doc->pages[page].boundingbox[URX] >
-	   doc->pages[page].boundingbox[LLX]) &&
-	  (doc->pages[page].boundingbox[URY] >
-	   doc->pages[page].boundingbox[LLY])) {
-         /* use page bbox */
-	 new_llx = doc->pages[page].boundingbox[LLX];
-	 new_lly = doc->pages[page].boundingbox[LLY];
-	 new_urx = doc->pages[page].boundingbox[URX];
-	 new_ury = doc->pages[page].boundingbox[URY];
-      } else if ((doc->boundingbox[URX] > doc->boundingbox[LLX]) &&
-		 (doc->boundingbox[URY] > doc->boundingbox[LLY])) {
-	 /* use doc bbox */
-	 new_llx = doc->boundingbox[LLX];
-	 new_lly = doc->boundingbox[LLY];
-	 new_urx = doc->boundingbox[URX];
-	 new_ury = doc->boundingbox[URY];
-      }
+      psgetpagebbox (doc, page, &new_urx, &new_ury, &new_llx, &new_lly);
    } else {
       if (new_pagesize < 0)
          new_pagesize = DEFAULT_PAGE_SIZE;
